﻿using System;
using System.IO;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Navigation;

using System.Collections.ObjectModel;

using ExtendSample.photo;
using ExtendSample.view;
using ExtendSample.service;
using ExtendSample.service.hatena;
using ExtendSample.service.photozou;


namespace ExtendSample
{
	public partial class Window1
	{

        //Active PhotoCircle instance
        private PhotoCircle activeCircle = null;
        private bool isFirstLoaded = false;

        /// <summary>
        /// Constructor
        /// </summary>
        public Window1()
        {
            this.InitializeComponent();
        }


        #region イベントハンドラ

        /// <summary>
        /// Windowロード完了　イベントハンドラ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            Storyboard openingAnime = (Storyboard)this.FindResource("Timeline_opening");
            openingAnime.Completed += new EventHandler(OpeningAnime_Completed);
            this.BeginStoryboard(openingAnime);
        }

        /// <summary>
        /// オープニングアニメーション完了　イベントハンドラ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OpeningAnime_Completed(object sender, EventArgs e)
        {
            searchPanel.KickOpeningSearch();
        }



        #region SearchPanel カスタムRoutedEvent

        /// <summary>
        /// 検索ボタンクリックイベント「SearchButtonClicked」　イベントハンドラ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SearchPanel_SearchButtonClicked(object sender, RoutedEventArgs e)
        {
            FrameworkElement feSource = e.Source as FrameworkElement;
            SearchPanel searchPanel = (SearchPanel)feSource;

            PhotoCircleHistoryAction();

            GetSearchResult();
        }


        /// <summary>
        /// 全消去ボタンクリックイベント「AllDeleteButtonClicked」　イベントハンドラ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SearchPanel_DeleteButtonClicked(object sender, RoutedEventArgs e)
        {
            int circleCount = mainCanvas.Children.Count;

            for (int i = 0; i < circleCount; i++)
            {
                PhotoCircle circle = (PhotoCircle)mainCanvas.Children[i];
                circle = null;
            }

            mainCanvas.Children.Clear();
        }

        #endregion


        #region From MyListPanel カスタムRoutedEvent


        /// <summary>
        /// リスト選択イベント「PhotoSelected」　イベントハンドラ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MyListPanel_PhotoSelected(object sender, RoutedEventArgs e)
        {
            FrameworkElement feSource = e.Source as FrameworkElement;
            MyListPanel myList = (MyListPanel)feSource;

            int selectNumber = myList.SelectPhotoNumber;
            Collection<Photo> activeCollection = (Collection<Photo>)myList.MyCollectionData;

            photoViewer.Show(activeCollection, selectNumber, "My List");
        }

        #endregion


        #region PhotoCircle カスタムRoutedEvent

        /// <summary>
        /// 初期アニメ終了イベント「OpenAnimmeCompleted」　イベントハンドラ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PhotoCircle_OpenAnimmeCompleted(object sender, RoutedEventArgs e)
        {
            if (isFirstLoaded == false)
            {
                searchPanel.KickOpenAnimation();
            }
            isFirstLoaded = true;
        }

        /// <summary>
        /// 閉じるボタンクリックイベント「CloseCircleClicked」　イベントハンドラ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PhotoCircle_CloseCircleClicked(object sender, RoutedEventArgs e)
        {
            FrameworkElement feSource = e.Source as FrameworkElement;
            PhotoCircle circle = (PhotoCircle)feSource;

            int circleCount = mainCanvas.Children.Count;

            for (int i = 0; i < circleCount; i++)
            {
                PhotoCircle c = (PhotoCircle)mainCanvas.Children[i];
                if (c == circle)
                {
                    mainCanvas.Children.RemoveAt(i);
                    break;
                }
            }
        }


        /// <summary>
        /// リスト選択イベント「PhotoSelected」　イベントハンドラ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PhotoCircle_PhotoSelected(object sender, RoutedEventArgs e)
        {
            FrameworkElement feSource = e.Source as FrameworkElement;
            PhotoCircle circle = (PhotoCircle)feSource;

            activeCircle = circle;

            int selectNumber = circle.SelectPhotoNumber;
            Collection<Photo> activeCollection = circle.ResultCollection;

            photoViewer.Show(activeCollection, selectNumber, "Search");
        }


        /// <summary>
        /// PhotoCircle小選択イベント「SmallCircleSelected」　イベントハンドラ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PhotoCircle_SmallCircleSelected(object sender, RoutedEventArgs e)
        {
            FrameworkElement feSource = e.Source as FrameworkElement;
            PhotoCircle circle = (PhotoCircle)feSource;

            //サムネイルサークルの数
            int circleCount = mainCanvas.Children.Count;

            //直前履歴 
            if (circleCount > 0)
            {
                PhotoCircle c = (PhotoCircle)mainCanvas.Children[circleCount - 1];
                c.Small();
            }

            //クリックされたサムネイルサークルを一度削除
            mainCanvas.Children.Remove(circle);

            //回転用にサムネイルサークルの数を再取得
            circleCount = mainCanvas.Children.Count;

            //履歴円を回転
            for (int i = 0; i < circleCount; i++)
            {

                PhotoCircle outerCircle = (PhotoCircle)mainCanvas.Children[i];

                if (i != circleCount - 1)
                {
                    outerCircle.Rotate(12 - 1 - i);
                }
                else
                {
                    double nowAngle = outerCircle.GetUserControlAngle();
                    if (nowAngle != 0)
                    {
                        outerCircle.Rotate(12 - 1 - i);
                    }
                }

            }

            //クリックされたサムネイルサークルを戻す
            mainCanvas.Children.Add(circle);
        }


        /// <summary>
        /// リストアイテム内ボタンクリックイベント「AddMyListButtonClicked」　イベントハンドラ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PhotoCircle_AddMyListButtonClicked(object sender, RoutedEventArgs e)
        {
            FrameworkElement feSource = e.Source as FrameworkElement;
            PhotoCircle currentCircle = (PhotoCircle)feSource;

            Photo photo = currentCircle.SelectedPhoto;

            myListPanel.OpenAnimation();
            myListPanel.AddPhoto(photo);
        }


        #endregion

        #endregion


        #region private Method

        /// <summary>
        ///サービスに接続し、結果を受け取って、サムネイルサークルを作成
        /// </summary>
        private void GetSearchResult()
        {
            Collection<Photo> resultCollection = new Collection<Photo>();

            //検索情報取得
            IService searchService = searchPanel.ActiveService;
            string searcedhWord = searchPanel.SearchWord;
            string serviceName = searchService.GetServiceName();

            //サービス接続とコレクション取得
            resultCollection = searchService.Send(searcedhWord);

            //サムネイルサークル作成
            AttacheCircle(resultCollection, serviceName, searcedhWord);

        }

        /// <summary>
        /// サムネイルサークル作成
        /// </summary>
        /// <param name="resultCollection"></param>
        /// <param name="serviceName"></param>
        /// <param name="searcedhWord"></param>
        private void AttacheCircle(Collection<Photo> resultCollection, string serviceName, string searcedhWord)
        {
            //サムネイルサークル作成
            PhotoCircle circle = new PhotoCircle(resultCollection, serviceName, searcedhWord);
            mainCanvas.Children.Add(circle);


            //Canvasサイズ取得
            double canvasCenterX = mainCanvas.ActualWidth;
            double canvasCenterY = mainCanvas.ActualHeight;

            //サムネイルサークルサイズ取得
            double circleWidth = circle.Width;
            double circleHeight = circle.Height;

            //サムネイルサークルの座標セット
            Canvas.SetLeft(circle, canvasCenterX / 2 - circleWidth / 2);
            Canvas.SetTop(circle, canvasCenterY / 2 - circleHeight / 2);
        }


        /// <summary>
        /// 履歴円のサムネイルサークル制御
        /// </summary>
        private void PhotoCircleHistoryAction()
        {
            int circleCount = mainCanvas.Children.Count;

            //直前履歴
            if (circleCount != 0)
            {
                PhotoCircle c = (PhotoCircle)mainCanvas.Children[circleCount - 1];
                c.Small();
            }

            
            if (circleCount >= 13)
            {
                //一番古いサムネイルサークルを削除
                PhotoCircle oldestCircle = (PhotoCircle)mainCanvas.Children[0];
                mainCanvas.Children.Remove(oldestCircle);
                oldestCircle = null;

                //サムネイルサークル数再取得
                circleCount = mainCanvas.Children.Count;

                //各履歴円を回転
                for (int i = 0; i < circleCount - 1; i++)
                {
                    PhotoCircle outerCircle = (PhotoCircle)mainCanvas.Children[i];
                    outerCircle.Rotate(circleCount - 1 - i);
                }
            }
            else
            {
                //直前の履歴円だったサムネイルサークルを回転
                if (circleCount > 1)
                {
                    PhotoCircle c = (PhotoCircle)mainCanvas.Children[circleCount - 2];
                    c.ArchivalRotate(12 - circleCount + 1);
                }
            }
        }



        #endregion

    }
}